using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.RSSBus.QuickBooks;

namespace cacheinvoicesDemo
{
  public partial class CacheInvoices : Form
  {
    private AutoUpdate autoupdate1 = new AutoUpdate();
    private BindingSource qbInvoiceSource = new BindingSource();
    private String connectionString = "";
    private String _tableName = "Invoices";
    private String _queryString = "";
    private QuickBooksDataAdapter dataAdapter = new QuickBooksDataAdapter();

    public CacheInvoices() {
      InitializeComponent();
      txtQuery.Text = "SELECT * FROM " + _tableName;
    }

    private void buildConnectionString(Boolean cached)
    {
      connectionString = "Application Name=RSSBus QuickBooks Data Provider;";
      if (!cached)
      {
          connectionString = connectionString + string.Format("User={0};Password={1};URL={2};Cache Location={3};Auto Cache=True;Offline=False;Query Passthrough=True;", txtUserName.Text, txtPassword.Text, txtURL.Text, txtCacheLoc.Text);
      }
      else 
      {
            connectionString = connectionString + string.Format("Cache Location={0};Auto Cache=True;Offline=True;Query Passthrough=True;", txtCacheLoc.Text);
      }        
    }

    private void createCache() 
    {
        buildConnectionString(false);
        _queryString = "SELECT * FROM " + _tableName + " WHERE StartModifiedDate = '"+DateTime.Today.Subtract(TimeSpan.FromDays(60)).ToShortDateString() + "'";
        executeQuery();
    }

    private void executeQuery() 
    {
        using (QuickBooksConnection conn = new QuickBooksConnection(connectionString)) 
        {

            dgvInvoices.DataSource = qbInvoiceSource;
            DataTable table = new DataTable();
            dataAdapter = new QuickBooksDataAdapter(_queryString, conn);
            dataAdapter.Fill(table);
            for (int i = 0; i < table.Columns.Count; i++)
            {
                if (table.Columns[i].ColumnName.Equals("ID") || table.Columns[i].ColumnName.Equals("CustomerName") ||
                    table.Columns[i].ColumnName.Equals("Items") || table.Columns[i].ColumnName.Equals("ReferenceNumber") ||
                    table.Columns[i].ColumnName.Equals("DueDate") || table.Columns[i].ColumnName.Equals("TimeModified"))
                {
                }
                else
                {
                    table.Columns[i].ColumnMapping = MappingType.Hidden;
                }
            }
            qbInvoiceSource.DataSource = table;
            dataAdapter.Dispose();
        }
        
    }

    private void btn_CreateCache_Click(object sender, EventArgs e)
    {
        Cursor.Current = Cursors.WaitCursor;
        createCache();
        Cursor.Current = Cursors.Default;
    }

    private void btn_ExecuteQuery_Click(object sender, EventArgs e)
    {
        Cursor.Current = Cursors.WaitCursor;
        if (txtQuery.Text != null && !txtQuery.Text.Equals(""))
        {
            _queryString = txtQuery.Text;
        }
        else
        {
            _queryString = "SELECT * FROM " + _tableName;
            txtQuery.Text = _queryString;
        }
        buildConnectionString(true);
        executeQuery();
        Cursor.Current = Cursors.Default;
    }

    private void btn_RefreshCache_Click(object sender, EventArgs e)
    {
        Cursor.Current = Cursors.WaitCursor;
        buildConnectionString(false);
        DataTable table = new DataTable();
        string connstr = connectionString.Replace("Offline=False", "Offline=True");
        connstr = connstr.Replace("Query Passthrough=False", "Query Passthrough=True");
        QuickBooksDataAdapter dataAdapter = new QuickBooksDataAdapter("SELECT MAX(TimeModified) as MaxTime FROM Invoices", connstr);
        dataAdapter.Fill(table);
        string date;
        try {
            date = ((string)table.Rows[0]["MaxTime"]);
        } catch (Exception ex) {
            MessageBox.Show(ex.Message);
            date = "1970-01-01";
        }
        table.Rows.Clear();
        table.Columns.Clear();
        dataAdapter.Dispose();
        connstr = connstr.Replace("Offline=True", "Offline=False");
        connstr = connstr.Replace("Query Passthrough=True", "Query Passthrough=False");
        QuickBooksConnection conn = new QuickBooksConnection(connstr);
        dataAdapter = new QuickBooksDataAdapter("SELECT * FROM Invoices WHERE StartModifiedDate = '" + date + "'", conn);
        dataAdapter.Fill(table);
        dataAdapter.Dispose();
        conn.Close();

        btn_ExecuteQuery_Click(null, null);
        Cursor.Current = Cursors.Default;
    }
  }

  static class Program
  {
    [STAThread]
    static void Main()
    {
      Application.EnableVisualStyles();
      Application.SetCompatibleTextRenderingDefault(false);
      Application.Run(new CacheInvoices());
    }
  }
}

















